@@ -2,7 +2,9 @@
 read.adp.sontek <- function(file, from=1, to, by=1, tz=getOption("oceTz"),
                             latitude=NA, longitude=NA,
                             type=c("adp", "pcadp"),
-                            debug=getOption("oceDebug"), monitor=FALSE, despike=FALSE, processingLog, ...)
+                            monitor=FALSE, despike=FALSE, processingLog,
+                            debug=getOption("oceDebug"),
+                            ...)
 {
     missing.to <- missing(to)
     ## In this function, comments in [] refer to logical page number of ADPManual_v710.pd; add 14 for file page number
