@@ -2,6 +2,8 @@ pwelch <- function(x, window, noverlap, nfft, fs, spectrumtype, esttype,
                    plot=TRUE,
                    debug=getOption("oceDebug"), ...)
 {
+    ##http://octave.svn.sourceforge.net/viewvc/octave/trunk/octave-forge/main/signal/inst/pwelch.m
+
     hamming.local <- function (n) # avoid having to pull in the signal library
     {
         n <- round(n)
@@ -32,9 +34,11 @@ pwelch <- function(x, window, noverlap, nfft, fs, spectrumtype, esttype,
     gave.window <- !missing(window)
     gave.nfft <- !missing(nfft)
     gave.fs <- !missing(fs)
-    oceDebug(debug, sprintf("\b\bpwelch(x, window=%s, nfft=%s, fs=%s, ...) {\n",
+    gave.noverlap <- !missing(noverlap)
+    oceDebug(debug, sprintf("\b\bpwelch(x, window=%s, nfft=%s, fs=%s, noverlap=%s, ...) {\n",
                              if (gave.window) window else "(not given)",
                              if (gave.nfft) nfft else "(not given)",
+                             if (gave.noverlap) noverlap else "(not given)",
                              if (gave.fs) fs else "(not given)"))
     if (is.ts(x)) {
         if (missing(fs))
@@ -69,7 +73,12 @@ pwelch <- function(x, window, noverlap, nfft, fs, spectrumtype, esttype,
                 nfft <- x.len
             window <- hamming.local(nfft)
         } else {
-            window <- hamming.local(floor(x.len / 8))
+            if (gave.noverlap) {
+                windowLength <- min(x.len, floor(x.len / 8))
+            } else {
+                windowLength <- min(x.len, floor(x.len / 8 / 0.5))
+            }
+            window <- hamming.local(windowLength)
         }
     }
     normalization <- mean(window^2)
@@ -100,7 +109,7 @@ pwelch <- function(x, window, noverlap, nfft, fs, spectrumtype, esttype,
     while (TRUE) {
         oceDebug(debug, "  subspectrum at indices ", start, "to", end, "\n")
         xx <- ts(window * detrend(x[start:end]), frequency=fs)
-        args$x <- xx
+        args$x <- as.vector(xx)
         s <- do.call(spectrum, args=args)
         if (nrow == 0)
             freq <- s$freq
@@ -117,7 +126,8 @@ pwelch <- function(x, window, noverlap, nfft, fs, spectrumtype, esttype,
     oceDebug(debug, "\b\b} # pwelch()\n")
     rval <- list(freq=freq, spec=apply(psd, 2, mean), 
                  method="Welch", series=deparse(substitute(x)),
-                 df=s$df, bandwidth=s$bandwidth, # FIXME: wrong formulae
+                 df=s$df * (x.len / length(window)),
+                 bandwidth=s$bandwidth, # FIXME: wrong formulae
                  demean=FALSE, detrend=TRUE)
     class(rval) <- "spec"
     if (plot) {
